#include "kblinselstereogram.h"

KBlinselStereogram::KBlinselStereogram()
{
  use_texturing = false;
}

KBlinselStereogram::~KBlinselStereogram()
{
}

void KBlinselStereogram::setMaxDistance(double maxDist)
{
depf=maxDist/256.;
}



void KBlinselStereogram::init_stg()
{
 int x,y;
 maxX = source.width();
 maxY = source.height();
 y = 0;
 x = 0;
 unsigned int maxdepthused=255;
 for (x=0; x<maxX; x++)
  for (y=0; y<maxY; y++)
   if (source.pixel(x,y) < maxdepthused) maxdepthused=source.pixel(x,y);
 mdu = 
                 depf * (float)
                 ((qRed  (maxdepthused)+
                   qGreen(maxdepthused)+
                   qBlue (maxdepthused)) / 3.);  
 maxsep = round((1-mu*mdu)*E/(2-mu*mdu));
}

void KBlinselStereogram::init_field()
{
 int i,x,y;
 Z=malloc( (source.width()+1+maxsep) * sizeof(float*) );
 for (i=0; i<source.width()+1+maxsep; i++)
  Z[i]=malloc( (source.height()+1)*sizeof(float) ); 

 for (x=0; x<maxX+maxsep; x++)
  for (y=0; y<maxY; y++) *(Z[x]+y)=mdu;

 for(y = 0; y < maxY; y++) 
  for(x = 0; x < maxX; x++)
   *(Z[x+(maxsep/2)]+y)=
   depf * (float)
   ((qRed  (source.pixel(x,y))+
     qGreen(source.pixel(x,y))+
     qBlue (source.pixel(x,y)) ) / 3.);  
}

void KBlinselStereogram::init_texture()
{
 int i,x,y;
 textX = pattern.height(); 
 textY = pattern.width(); 
 if (use_texturing==false)
 { textX=maxX+maxsep; textY=maxY; }

 texture=malloc(textX*sizeof(int*));
 for (i=0; i<textX; i++)
  texture[i]=malloc(textY*sizeof(int));

/* Konvertierung */
 source = source.convertDepth(8);

 img1.create(maxX+maxsep,maxY,8);

 if(use_texturing == true)
 {
 pattern = pattern.convertDepth(8); 
 pattern.setNumColors(256); 
 img1.setNumColors(256);
  for(i = 0; i < 256; i++)
  {
   img1.setColor(i, pattern.color(i)); 
  }     
 img1.convertDepth(8);
 for(y = 0; y < textY; y++)
  for(x = 0; x < textX; x++) 
   *(texture[x]+y) = pattern.pixelIndex(x,y);
 }  
 else   
 {
  img1.setNumColors(2);
  img1.setColor(0,qRgb(0,0,0));
  img1.setColor(1,qRgb(255,255,255));
  for (x=0; x<maxX+maxsep; x++)
   for (y=0; y<maxY; y++)
    *(texture[x]+y)=random()&1;  
 }
}


QImage KBlinselStereogram::calc()
{
/* Berechnung */
int x, y;

 for(y=0; y < maxY; y++) 
    {
     int *point = new int[maxX+maxsep]; 
     int *sim = new int[maxX+maxsep];
     int s;
     int left,right;

     emit renderStatus(y); 

     for( x = 0; x < maxX+maxsep; x++ ) *(sim+x) = x;

     for( x = 0; x < maxX+maxsep; x++ ) 
     {
       s = round((1-mu* (*(Z[x]+y)) )*E/(2-mu* (*(Z[x]+y)) ));
       left = x - (s/2);
       right = left + s;        
       if ( 0 <= left && right < maxX+maxsep )
       {       
        { 
          int k;                
          for(k=sim[left]; k!=left && k!=right; k=*(sim+left))
           if( k < right ) left = k;
           else { left = right; right = k; }
           *(sim+left) = right; 
        }
       }
     }

     for( x = (maxX+maxsep)-1; x >= 0; x-- ) 
      {
       if( *(sim+x) == x ) 
        *(point+x) = *(texture[x%textX]+(y%textY));        
       else 
        *(point+x) = *(point+*(sim+x)); 
       img1.setPixel(x,y,*(point+x));
      }

 free(point);
 free(sim);
 }

for (x=0; x<source.width()+1+maxsep; x++)
 free(Z[x]);
free(Z);

for (x=0; x<textX; x++)
 free(texture[x]);
free(texture);

return img1;
}



// Only a first try !
// Doesn't work at all.
/*
QImage KBlinselStereogram::calcback()
{
QImage stgr;
const int maxX = source.width();
const int maxY = source.height();
int line, x, y, mp, sep;
float outd[maxX][maxY];
float eyedst, eyesep, depth, maxdepth;
bool found;

eyedst=5;
eyesep=9;
maxdepth=0;

source = source.convertDepth(8);
stgr.convertDepth(8);
stgr.create(maxX,maxY,8);
stgr.setNumColors(256);

for (x=0; x<256; x++)
 stgr.setColor(x, qRgb(x, x, x));

for (line=0; line<maxY; line++) {
 for (mp=0; mp<maxX; mp++)
  {
  found=FALSE;
  sep=0;
  while (found==FALSE && sep < (int) eyedst)
   {
   sep++;
   if ( mp - sep >= 0) 
   {if ( ( source.pixelIndex(mp+sep,line)) == 
        ( source.pixelIndex(mp-sep,line)) )
       found=TRUE;} else
   {found=TRUE; sep=0;}           
   }

  if (found==FALSE) {depth=0;} else
   depth=eyedst / ((eyesep/(float)sep) - 1.);
  if (depth<0) depth=0;

  if (depth>maxdepth) maxdepth=depth;

  outd[mp][line]=depth;
  }   
 } 

for (x=0; x<maxX; x++)
 for (y=0; y<maxY; y++)
  stgr.setPixel(x,y,(int) ((float)outd[x][y]*(256./(float)maxdepth)) );

return stgr;
}
*/

void KBlinselStereogram::setSource(QImage &img)
{
 source = img;
}

void KBlinselStereogram::setTexture(QImage &texture)
{
 pattern = texture;
}
